#include <fggl/scenes/menu.hpp>
#include <fggl/util/service.hpp>
#include <fggl/gui/gui.hpp>

#include <spdlog/spdlog.h>

namespace fggl::scenes {

    using fggl::input::MouseButton;
    using fggl::input::MouseAxis;



    BasicMenu::BasicMenu(fggl::App& app) : AppState(app), m_inputs(nullptr), m_active(), m_hover(nullptr) {
        auto& locator = fggl::util::ServiceLocator::instance();
        m_inputs = locator.get<input::Input>();
    }

    void BasicMenu::update() {
        if ( m_inputs != nullptr ) {
            m_cursorPos.x = m_inputs->mouse.axis( MouseAxis::X );
            m_cursorPos.y = m_inputs->mouse.axis( MouseAxis::Y );

			// in canvas space
			math::vec2 projected;
			projected.x = math::rescale_ndc(m_cursorPos.x, 0, 1920.f);
			//projected.y = math::rescale_ndc(m_cursorPos.y, 1080.0f, 0);
			projected.y = math::rescale_ndc(m_cursorPos.y, 0, 1080.0f);

			auto* hoverWidget = m_canvas.getChildAt(projected);
			if ( hoverWidget != m_hover ){
				if ( m_hover != nullptr ) {
					m_hover->onExit();
				}
				m_hover = hoverWidget;
				if ( m_hover != nullptr ){
					m_hover->onEnter();
				}
			}

			if ( m_inputs->mouse.pressed( MouseButton::LEFT ) ) {
				spdlog::info("clicky clicky: ({}, {})", projected.x, projected.y);

				auto widget = m_canvas.getChildAt(projected);
				if (widget != nullptr) {
					widget->activate();
					spdlog::info("ooo! there is a thing there!");
				}
            }
        }
    }

    void BasicMenu::render(gfx::Graphics& gfx) {
		// render the 2D scene (we don't have a 3D scene to worry about in menus)
		gfx::Paint paint;
		m_canvas.render( paint );
		gfx.draw2D(paint);
    }

    void BasicMenu::activate() {

    }

    void BasicMenu::deactivate() {

    }

    void BasicMenu::add(const std::string& name, callback cb) {
        m_items[name] = cb;

		const math::vec2 btnSize{ 150.0f, 30.0f };
		const float spacing = 5;
		const float padX = 50.0f;
		const float padY = 50.0f;

		// figure out the position based off the old logic
		// FIXME should be the container's job
		math::vec2 pos { 1920.0f - ( padX + btnSize.x ), padY };
		auto btnIdx = m_items.size() - 1;
		pos.y += (btnIdx * (btnSize.y + spacing));

		// build the button
		auto btn = std::make_unique<gui::Button>(pos, btnSize);
		btn->label(name);

		btn->addCallback(cb);
		m_canvas.add(std::move(btn));
    }

};
